/************************************************************************************************************************/
/*

Software for smart cameras from Vision Components
-------------------------------------------------

Program: Klaus Schneider, VC

Program Description:
--------------------



*/
/************************************************************************************************************************/

////////////////////////////////////////////////////////////////////////////////////////
// TESTING VERSION with telnet output, auto trigger and modul display 
////////////////////////////////////////////////////////////////////////////////////////

// no VCRT version?
// #define VCRT_NO_COMPAT

// #define   TESTING_VERSION
#define   CHECK_LICENCE_CODE

// WIBU Licence check
// #define WIBU_LICENCE

/************************************************************************************************************************/

#define NEW_IMAGE_VAR

#include <float.h>
#include <mathf.h>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


#ifdef VCRT_NO_COMPAT
	#include "\ti\vclib\licence.h"
  #define  sprint  sprintf
	#include "\ti\include\macros.h"
	#include "\ti\include\vcrt.h"
	#include "\ti\include\vclib.h"
	#include "\ti\include\flib.h"
	#include "\ti\include\register.h"
	#include "\ti\include\sysvar.h"
#else
	#include <licence.h>
	#include <vcrt.h>
	#include <flib.h>
	#include <vclib.h>
	#include <macros.h>
	#include <sysvar.h>
#endif

/************************************************************************************************************************/

#define  EVEN(x)             ((x)&0xFFFFFFFE)
#define  Round(x)            ((I32)floorf(x+0.5))

#define  FIND_PAT            0
#define  DELETE_PAT          1
#define  NEXT_PAT            2
#define  FREE_MEM            3   // release temporary memory. NEXT_PAT or DELETE_PAT is not possible any more. Same is done at function DeinitSmartFinder()

/************************************************************************************************************************/

typedef struct
{
  //////////////////////////////////////////////////
  // input parameter
  //////////////////////////////////////////////////

  // please set the variable LicenceCode to the value offered from Vision Components
  U32   LicenceCode1;    // hex value like 0x1234ABCD
  U32   LicenceCode2;    // hex value like 0xABCD1234

  // AngleMin [-360..360] and AngleMax [AngleMin..AngleMin+360] (for example -45 till +45 degrees: AngleMin=-45 AngleMax=45). 
  I32   AngleMin;
  I32   AngleMax;
  I32   AngleStep;

  // Scaling Range in percent [-50%..50%] 
  I32   ScaleMin;
  I32   ScaleMax;
  I32   ScaleStep;
  

  
  //////////////////////////////////////////////////
  // standard search parameter
  //////////////////////////////////////////////////

  // pyramid zoom for search area (speed improvements)
  I32   PyrZoom;

  // SearchAcc[] (0 = Low / 1 = High Subpixel and Pattern Accuracy)
  // [0]: Search Step
  // [1]: Adjust Step
  I32   SearchAcc[2];
  
  // SpeedLevel: [0 = Safe .. 9 = Fast]
  I32   SpeedLevel;



  //////////////////////////////////////////////////
  // expert parameter
  //////////////////////////////////////////////////

  // Expert Mode (0 = Overwrite Expert Parameter depending on SpeedLevel / 1 = Use Expert Parameter)
  I32   ExpertMode;

  // edge detection parameter for VC function edge()
  I32   Type;
  I32   Sigma;
  I32   BinMode;
  I32   MinContr;
  I32   Thresh;
  
  // Define Pattern Anchor Point Type
  // Typ  1: Centre of Pattern Bounding Box (recommended)
  // Typ  2: Centre of Pattern Gravity
  // Typ  0: Use defined AnchorX and AnchorY Points inside Pattern Area (experienced user only)
  I32   AnchorTyp;
  I32   AnchorX;
  I32   AnchorY;

  // define distance between contour pixels for speed improvements
  // [0]: Search Step
  // [1]: Adjust Step
  I32   PatRes[2];
  I32   SrcRes[2];

  // Memory Level [1 = high level .. 3 = low level]
  I32   MemLevel;

  // Memory Mode (1 = cache optimized / 0 = slow but higher resolution)
  I32   MemMode;

  

  //////////////////////////////////////////////////
  // teached pattern parameter
  //////////////////////////////////////////////////

  // pattern data to store
  I32   PatternLen;
  U8   *PatternData;

  

  //////////////////////////////////////////////////
  // output parameters
  //////////////////////////////////////////////////

  float PosX;
  float PosY;
  float Phi;
  float Scale;
  float Match;
  
  I32   BoundingX;
  I32   BoundingY;
  I32   BoundingDx;
  I32   BoundingDy;
  
  I32   PatListLenXY;

  I32   Error;
  
  // smart finder lib version (18 = version 1.8)
  I32   LibVersion;


  //////////////////////////////////////////////////
  // for internal use only
  //////////////////////////////////////////////////

  I32   *MemPointer;
} 

PatternPar;


/************************************************************************************************************************/
/************************************************************************************************************************/
/************************************************************************************************************************/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION  
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/************************************************************************************************************************/
/************************************************************************************************************************/
/************************************************************************************************************************/


/************************************************************************************************************************/

// DM_ReaderDemo.c    

I32  TeachPattern              (PatternPar *PatPar, image *Pat);

void IniCamera                 (I32 *ScreenX, I32 *ScreenY, I32 *ScreenDx, I32 *ScreenDy);
void IniCameraBack             (void);

I32  SetParameter              (PatternPar *PatPar);

void ChangeParameter           (PatternPar *PatPar);
void ChangeValue               (I32 *ParValue, I32 Min, I32 Max, I32 Step, I8 *text);

I32  StorePattern              (PatternPar *PatPar, char *FileName);
I32  ReadPattern               (PatternPar *PatPar, char *FileName);

int  ReadKeyboard              (I32 wait);
void EmptyKeyboardBuffer       (void);

void SelectArea                (I32 *px, I32 *py, I32 *pdx, I32 *pdy, I32 xmin, I32 xmax, I32 ymin, I32 ymax, I32 text, I32 even, I32 color);

void CapturePosition           (I32 x, I32 y);

void ChangeColor               (image *area, I32 start, I32 end, I32 newcol);

void High                      (image *Area);
void Low                       (image *Area);

void EmptyKeyboardBufferAll    (void);

/************************************************************************************************************************/

// SmartFinder LIB FUNCTIONS

I32  InitSmartFinder           (PatternPar *PatPar);
I32  TeachPatternEdge          (PatternPar *PatPar, image *ImgEdge);
I32  TeachPatternData          (PatternPar *PatPar, U8 *PatData, I32 PatLen);
I32  TeachParameterChange      (PatternPar *PatPar);
I32  FindPattern               (PatternPar *PatPar, image *ImgSearch, I32 Mode);
I32  DrawPattern               (PatternPar *PatPar, image *DrawArea, I32 Color);
I32  ListPattern               (PatternPar *PatPar, I32 *PatXY);
I32  DeinitSmartFinder         (PatternPar *PatPar);

/************************************************************************************************************************/


